import uuid

from datetime import datetime
from typing import TYPE_CHECKING, Optional

from barker.models.guest_book_type import GuestBookType
from sqlalchemy import (
    CheckConstraint,
    DateTime,
    Enum,
    ForeignKey,
    Integer,
    Text,
    Uuid,
    text,
)
from sqlalchemy.orm import Mapped, mapped_column, relationship

from ..db.base_class import reg


if TYPE_CHECKING:
    from .customer import Customer
    from .overview import Overview


@reg.mapped_as_dataclass(unsafe_hash=True)
class GuestBook:
    __tablename__ = "guest_book"
    __table_args__ = (CheckConstraint("num_nonnulls(booking_date, arrival_date) > 0", name="ck_guest_book_nulls"),)

    id: Mapped[uuid.UUID] = mapped_column(Uuid, primary_key=True, server_default=text("gen_random_uuid()"))
    customer_id: Mapped[uuid.UUID] = mapped_column(Uuid, ForeignKey("customers.id"), nullable=False)
    pax: Mapped[int] = mapped_column(Integer, nullable=False)
    booking_date: Mapped[Optional[datetime]] = mapped_column(DateTime(), nullable=True)
    arrival_date: Mapped[Optional[datetime]] = mapped_column(DateTime(), nullable=True)
    last_edit_date: Mapped[datetime] = mapped_column(DateTime(), nullable=False)
    notes: Mapped[str | None] = mapped_column(Text, nullable=True)
    type_: Mapped[GuestBookType] = mapped_column("type", Enum(GuestBookType), nullable=False)

    customer: Mapped["Customer"] = relationship("Customer")
    status: Mapped[Optional["Overview"]] = relationship(back_populates="guest", uselist=False)

    def __init__(
        self,
        customer_id=None,
        pax=None,
        booking_date=None,
        arrival_date=None,
        notes=None,
        type_=None,
        customer=None,
        id_=None,
    ):
        if customer_id is not None:
            self.customer_id = customer_id
        self.pax = pax
        if booking_date is None and arrival_date is None:
            raise Exception("Both arrival and booking date cannot be null")

        self.booking_date = booking_date
        self.arrival_date = arrival_date
        self.last_edit_date = datetime.utcnow()
        self.notes = notes
        self.type_ = type_
        if customer is None:
            self.customer_id = customer_id
        else:
            self.customer = customer
        self.id = id_
